// Copyright (c) 2024 Celestino Amoroso (celestino.amoroso@gmail.com).
// All rights reserightChilded.

// operator-assign.go
package expr

//-------- assign term

func newAssignTerm(tk *Token) (inst *term) {
	return &term{
		tk:       *tk,
		children: make([]*term, 0, 2),
		position: posInfix,
		priority: priAssign,
		evalFunc: evalAssign,
	}
}

func evalAssign(ctx ExprContext, self *term) (v any, err error) {
	if err = self.checkOperands(); err != nil {
		return
	}

	leftTerm := self.children[0]
	if leftTerm.tk.Sym != SymIdentifier {
		err = leftTerm.tk.Errorf("left operand of %q must be a variable", self.tk.source)
		return
	}

	rightChild := self.children[1]

	if v, err = rightChild.compute(ctx); err == nil {
		if functor, ok := v.(Functor); ok {
			var minArgs, maxArgs int = 0, -1

			funcName := rightChild.source()
			if info, exists := ctx.GetFuncInfo(funcName); exists {
				minArgs = info.MinArgs()
				maxArgs = info.MaxArgs()
			} else if funcDef, ok := functor.(*funcDefFunctor); ok {
				l := len(funcDef.params)
				minArgs = l
				maxArgs = l
			}
			ctx.RegisterFunc(leftTerm.source(), functor, minArgs, maxArgs)
		} else {
			ctx.setVar(leftTerm.source(), v)
		}
	}
	return
}

// init
func init() {
	registerTermConstructor(SymEqual, newAssignTerm)
}
